package org.ow2.petals.binding.soap;

/**
 * @author chamerling - eBM WebSourcing
 * 
 */
public final class Constants {

    public static class Component {
        public static final String NS_PREFIX = "soapbc";

        public static final String NS_URI = "http://petals.ow2.org/ns/soapbc";
    }

    /**
     * Axis2 related constants
     * 
     * @author Christophe HAMERLING - eBM WebSourcing
     * 
     */
    public static class Axis2 {

        public static final String AXIS2_XML = "axis2.xml";

        public static final String SERVICES_XML = "services.xml";

        public static final String MODULE_ARCHIVE_EXTENSION = "mar";

        public static final String MODULES_PATH = "modules";

        public static final String SERVICES_PATH = "services";

        public static final String ADDRESSING_MODULE = "addressing";
        
        public static final String RAMPART_MODULE = "rampart";

    }

    /**
     * JBI message related constants
     * 
     * @author Christophe HAMERLING - eBM WebSourcing
     * 
     */
    public static class JbiMessage {
        private static final String WS_PREFIX = "org.ow2.petals.ws";

        public static final String SOAP_SECURITY = WS_PREFIX + ".security";

        public static final String SOAP_VERSION = WS_PREFIX + ".version";

        public static final String SOAP_HEADER = WS_PREFIX + ".header";

        public static final String SOAP_USERNAME = WS_PREFIX + ".username";

        public static final String SOAP_PASSWORD = WS_PREFIX + ".password";
    }

    /**
     * Service Unit related constants
     * 
     * @author Christophe HAMERLING - eBM WebSourcing
     * 
     */
    public static class ServiceUnit {
        
        public static final String SOAP_ACTION = "soap-action";

        public static final String ADDRESS = "address";

        public static final String MODE = "mode";

        /**
         * The modules to be used as CSV. The Addressing module is always
         * included by default.
         */
        public static final String MODULES = "modules";

        /**
         * The service parameters, used to configure service.
         */
        public static final String SERVICE_PARAMETERS = "service-parameters";

        /**
         * The soap version of the outgoing message. Default is 1.1.
         */
        public static final String SOAP_VERSION = "soap-version";

        public static final String CLEANUP_TRANSPORT = "cleanup-transport";

        public static final String CHUNKED_MODE = "chunked-mode";

        public static final String PROXY_HOST = "proxy-host";

        public static final String PROXY_PORT = "proxy-port";

        public static final String PROXY_USER = "proxy-user";

        public static final String PROXY_PASSWORD = "proxy-password";

        public static final String PROXY_DOMAIN = "proxy-domain";

        /**
         * Basic authentication username
         */
        public static final String BASIC_AUTH_USERNAME = "authentication-basic-username";

        /**
         * Basic authentication password
         */
        public static final String BASIC_AUTH_PASSWORD = "authentication-basic-password";

        /**
         * Add a root element on outgoing soap messages.
         */
        public static final String ADD_ROOT = "add-root";

        /**
         * Remove the root element from incoming soap messages before building
         * the normalized message.
         */
        public static final String REMOVE_ROOT = "remove-root";

        /**
         * The HTTP method to be used for outgoing calls (Possible values are
         * GET, POST, PUT, DELETE).
         */
        public static final String REST_HTTP_METHOD = "rest-http-method";

        /**
         * 
         */
        public static final String REST_ADD_NS_URI = "rest-add-namespace-uri";

        /**
         * 
         */
        public static final String REST_ADD_NS_PREFIX = "rest-add-namespace-prefix";

        /**
         * 
         */
        public static final String REST_REMOVE_NS_PREFIX_RESP = "rest-remove-prefix-on-response";
        
        /**
         * 
         */
        public static final String HEADERS_FILTER = "headers-filter";
        
        /**
         * 
         */
        public static final String INJECT_HEADERS = "inject-headers";



        /**
         * A QName as string.
         */
        public static final String QNAME = "qname";

        /**
         * Timeout for synchronous request
         */
        public static final String TIMEOUT = "synchronous-timeout";
        
        // DEFAULTS
        public static final String DEFAULT_MODE = "SOAP";
        
        /**
         * The policy path is relative to the service unit root path.
         */
        public static final String POLICY_PATH = "policy-path";
        
    }
    
    /**
     * Policy constants
     * 
     * @author Christophe HAMERLING - eBM WebSourcing
     *
     */
    public static class Policy {
        public static final String POLICY_FILE = "policy.xml";
    }

    /**
     * HTTP server related constants
     * 
     * @author Christophe HAMERLING - eBM WebSourcing
     * 
     */
    public static class HttpServer {
        /**
         * The HTTP port the Jetty server listen incoming request on
         */
        public static final String HTTP_PORT = "http-port";

        /**
         * The interface address to be used for the server
         */
        public static final String HTTP_HOSTNAME = "http-host";

        /**
         * Provide or not services list
         */
        public static final String HTTP_SERVICES_LIST = "http-services-list";

        /**
         * The URL context
         */
        public static final String HTTP_SERVICES_CONTEXT = "http-services-context";

        /**
         * The service mapping URL
         */
        public static final String HTTP_SERVICES_MAPPING = "http-services-mapping";

        /**
         * The max size of the HTTP pool
         */
        public static final String HTTP_THREAD_POOL_SIZE_MAX = "http-thread-pool-size-max";

        /**
         * The min size of the HTTP pool
         */
        public static final String HTTP_THREAD_POOL_SIZE_MIN = "http-thread-pool-size-min";

        /**
         * The number of HTTP acceptors
         */
        public static final String HTTP_ACCEPTORS = "http-acceptors";

        /* Default values */

        public static final int DEFAULT_HTTP_PORT = 8084;
        
        public static final String DEFAULT_HTTP_HOST = "localhost";

        public static final boolean DEFAULT_HTTP_SERVICES_LIST = true;

        public static final String DEFAULT_PROTOCOL = "http";

        public static final String DEFAULT_HTTP_SERVICES_CONTEXT = "petals";

        public static final String DEFAULT_HTTP_SERVICES_MAPPING = "services";

        public static final int DEFAULT_HTTP_THREAD_POOL_SIZE_MAX = 255;

        public static final int DEFAULT_HTTP_THREAD_POOL_SIZE_MIN = 2;

        public static final int DEFAULT_HTTP_ACCEPTORS = 4;
    }

    public static class SOAP {
        public static final String ERROR_WRONG_MESSAGE_STATUS = "JBI message has wrong status";

        public static final String FAULT_SERVER = "Server";

        public static final String FAULT_CLIENT = "Client";
    }
}
