/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.binding.soap;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import org.apache.axiom.om.OMAttribute;
import org.apache.axiom.om.OMElement;
import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.deployment.DeploymentConstants;
import org.apache.axis2.deployment.DeploymentErrorMsgs;
import org.apache.axis2.deployment.DeploymentException;
import org.apache.axis2.deployment.ServiceBuilder;
import org.apache.axis2.description.AxisModule;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.axis2.i18n.Messages;
import org.apache.axis2.util.XMLUtils;
import org.ow2.petals.binding.soap.listener.SUPropertiesHelper;
import org.ow2.petals.binding.soap.listener.incoming.PetalsAxisService;
import org.ow2.petals.binding.soap.listener.incoming.PetalsReceiver;
import org.ow2.petals.binding.soap.listener.incoming.SoapServerConfig;
import org.ow2.petals.binding.soap.util.SoapSUClassLoader;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.su.ServiceUnitListener;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;
import org.ow2.petals.component.framework.su.ServiceUnitManager;
import org.ow2.petals.component.framework.util.ClassLoaderUtil;
import org.ow2.petals.ws.fault.WsnFault;
import org.ow2.petals.ws.notification.WsnManager;
import org.ow2.petals.ws.topic.Topic;

import static org.ow2.petals.binding.soap.Constants.Axis2.SERVICES_XML;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.MODULES;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.POLICY_PATH;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.SERVICE_PARAMETERS;

/**
 * A service unit listener used to register new service into Axis Engine during
 * SU deployment.
 * 
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public class SoapSUListener implements ServiceUnitListener {

    /**
     * The JBI component context
     */
    private final ComponentContext componentContext;

    /**
     * The SOAP component context (configuration values)
     */
    private final SoapComponentContext soapContext;

    /**
     * JDK logger
     */
    private final Logger logger;

    /**
     * 
     */
    private final SoapServerConfig soapServerConfig;

    private final PetalsReceiver petalsReceiver;

    private final AbstractComponent component;

    /**
     * Creates a new instance of {@link SoapSUListener}
     * 
     * @param soapContext
     * @param axisConfigurationContext
     * @param componentContext
     * @param servicesURL
     * @param logger
     */
    public SoapSUListener(final AbstractComponent component,
            final SoapComponentContext soapContext, final ComponentContext componentContext,
            final SoapServerConfig soapServerConfig, final PetalsReceiver petalsReceiver,
            final Logger logger) {
        this.component = component;
        this.soapContext = soapContext;
        this.componentContext = componentContext;
        this.soapServerConfig = soapServerConfig;
        this.petalsReceiver = petalsReceiver;
        this.logger = logger;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.su.ServiceUnitListener#onSUDeployed
     *      (java.lang.String, java.lang.String,
     *      org.ow2.petals.jbi.descriptor.JBIDescriptor)
     */
    public void onSUDeployed(final String serviceUnitName, final String suRootPath,
            final Jbi descriptor) throws PEtALSCDKException {
        this.logger.log(Level.FINE, "Deploying a new AxisService for SU " + serviceUnitName);
        
        // Global service unit configuration
        this.soapContext.addJbiDescriptor(serviceUnitName, descriptor);

        final File servicesXml = new File(suRootPath, SERVICES_XML);
        if (servicesXml.exists()) {
            this.soapContext.addServiceDescriptor(serviceUnitName, servicesXml);
        }
        
        // Local service configuration
        this.createServicesContext(serviceUnitName, suRootPath, descriptor);
    }
    
    /**
     * Create the services context
     * 
     * @param serviceUnitName
     * @param suRootPath
     * @param descriptor
     */
    private void createServicesContext(final String serviceUnitName, final String suRootPath,
            final Jbi descriptor) {
        ServiceUnitDataHandler suDatahandler = null;

        final List<Provides> providesList = descriptor.getServices().getProvides();
        for (final Provides provides : providesList) {
            if (suDatahandler == null) {
                suDatahandler = ((ServiceUnitManager) this.component.getServiceUnitManager())
                        .getSUDataHandlerForService(provides);
            }
            String address = SUPropertiesHelper.getAddress(suDatahandler
                    .getConfigurationExtensions(provides));
            this.soapContext.createServiceContext(address);
            ConfigurationExtensions extensions = suDatahandler.getConfigurationExtensions(provides);
            
            this.addModules(extensions);
            this.setServiceParameters(extensions);
            this.setPolicyPath(suRootPath, extensions);
            this.setServiceClassloader(suRootPath, extensions);
        }

        final List<Consumes> consumesList = descriptor.getServices().getConsumes();
        for (final Consumes consumes : consumesList) {
            if (suDatahandler == null) {
                suDatahandler = ((ServiceUnitManager) this.component.getServiceUnitManager())
                        .getSUDataHandlerForService(consumes);
            }
            String address = SUPropertiesHelper.getAddress(suDatahandler
                    .getConfigurationExtensions(consumes));
            this.soapContext.createServiceContext(address);
            
            ConfigurationExtensions extensions = suDatahandler.getConfigurationExtensions(consumes);
            this.addModules(extensions);
            this.setServiceParameters(extensions);
            this.setPolicyPath(suRootPath, extensions);
            this.setServiceClassloader(suRootPath, extensions);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.su.ServiceUnitListener#onSUStarted
     *      (java.lang.String)
     */
    public void onSUStarted(final String serviceUnitName) throws PEtALSCDKException {

        // get descriptors
        final Jbi descriptor = this.soapContext.getJbiDescriptor(serviceUnitName);
        final File serviceDescription = this.soapContext.getServiceDescriptor(serviceUnitName);

        if (descriptor != null) {

            ServiceUnitDataHandler suDatahandler = null;
            /*
             * Register new Axis2 service only on consumes nodes. Create an
             * AxisService for each requested endpoint. If an enpoint has not
             * been found, simply log a severe error. TODO : If a service
             * creation fails, and if services have been already registered for
             * this service unit, unregister them.
             */
            final List<Consumes> consumesList = descriptor.getServices().getConsumes();
            for (final Consumes consumes : consumesList) {
                if (suDatahandler == null) {
                    suDatahandler = ((ServiceUnitManager) this.component.getServiceUnitManager())
                            .getSUDataHandlerForService(consumes);
                }
                this.createAxisService(consumes, serviceDescription, suDatahandler
                        .getConfigurationExtensions(consumes));
            }

            // create/restore topics
            final List<Provides> providesList = descriptor.getServices().getProvides();
            for (final Provides provides : providesList) {
                if (suDatahandler == null) {
                    suDatahandler = ((ServiceUnitManager) this.component.getServiceUnitManager())
                            .getSUDataHandlerForService(provides);
                }
                this.createWSAccess(suDatahandler.getConfigurationExtensions(provides));
                this.createTopic(suDatahandler.getConfigurationExtensions(provides));
            }
        }
    }
    
    /**
     * Create the service context which will be used in the listeners
     * 
     * @param descriptor
     */
    protected void createServiceContexts(Jbi descriptor) {
        ServiceUnitDataHandler suDatahandler = null;

        final List<Provides> providesList = descriptor.getServices().getProvides();
        for (final Provides provides : providesList) {
            if (suDatahandler == null) {
                suDatahandler = ((ServiceUnitManager) this.component.getServiceUnitManager())
                        .getSUDataHandlerForService(provides);
            }
            String address = SUPropertiesHelper.getAddress(suDatahandler
                    .getConfigurationExtensions(provides));
            this.soapContext.createServiceContext(address);
        }

        final List<Consumes> consumesList = descriptor.getServices().getConsumes();
        for (final Consumes consumes : consumesList) {
            if (suDatahandler == null) {
                suDatahandler = ((ServiceUnitManager) this.component.getServiceUnitManager())
                        .getSUDataHandlerForService(consumes);
            }
            String address = SUPropertiesHelper.getAddress(suDatahandler
                    .getConfigurationExtensions(consumes));
            this.soapContext.createServiceContext(address);
        }
    }
    
    /**
     * Create an external web service provider
     * 
     * @param provides
     */
    protected void createWSAccess(final ConfigurationExtensions extensions) {

        final String externalWS = SUPropertiesHelper.getAddress(extensions);
        final String mode = SUPropertiesHelper.getMode(extensions);

        this.logger.info("Providing access to external service '" + externalWS + "' in mode '"
                + mode + "'");
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.su.ServiceUnitListener#onSUStopped
     *      (java.lang.String)
     */
    public void onSUStopped(final String serviceUnitName) throws PEtALSCDKException {

        final Jbi descriptor = this.soapContext.getJbiDescriptor(serviceUnitName);
        if (descriptor != null) {
            ServiceUnitDataHandler suDatahandler = null;

            // delete registered axis services
            final List<Consumes> consumesList = descriptor.getServices().getConsumes();
            for (final Consumes consumes : consumesList) {
                if (suDatahandler == null) {
                    suDatahandler = ((ServiceUnitManager) this.component.getServiceUnitManager())
                            .getSUDataHandlerForService(consumes);
                }
                this.unregisterAxisService(suDatahandler.getConfigurationExtensions(consumes));
            }

            // unregister topic
            final List<Provides> providesList = descriptor.getServices().getProvides();
            for (final Provides provides : providesList) {
                if (suDatahandler == null) {
                    suDatahandler = ((ServiceUnitManager) this.component.getServiceUnitManager())
                            .getSUDataHandlerForService(provides);
                }
                this.deleteTopic(suDatahandler.getConfigurationExtensions(provides));
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.su.ServiceUnitListener#onSUUndeployed
     *      (java.lang.String, java.lang.String,
     *      org.ow2.petals.component.framework.jbidescriptor.generated.Jbi)
     */
    public void onSUUndeployed(final String serviceUnitName, final String suRootPath,
            final Jbi descriptor) throws PEtALSCDKException {

        ServiceUnitDataHandler suDatahandler = null;

        // remove the topic persistance
        final List<Provides> providesList = descriptor.getServices().getProvides();
        for (final Provides provides : providesList) {
            if (suDatahandler == null) {
                suDatahandler = ((ServiceUnitManager) this.component.getServiceUnitManager())
                        .getSUDataHandlerForService(provides);
            }
            this.unregisterTopic(suDatahandler.getConfigurationExtensions(provides));
        }

        this.cleanServiceContexts(serviceUnitName, descriptor);
    }

    /**
     * Create the Axis service that will handle incoming SOAP calls.
     * TODO : Must be refactored into separate methods.
     * 
     * @param consumes
     * @param serviceDescriptor
     * @throws PEtALSComponentSDKException
     */
    private void createAxisService(final Consumes consumes, final File serviceDescriptor,
            final ConfigurationExtensions extensions) throws PEtALSCDKException {
        AxisService axisService = null;

        // Get the created service name
        final String newServiceName = SUPropertiesHelper.getAddress(extensions);

        if (newServiceName == null) {
            throw new PEtALSCDKException(
                    "Can not create an Axis service with a null name, the SU should provide an address extension value");
        }

        final AxisConfiguration axisConfig = this.soapContext.getAxis2ConfigurationContext()
                .getAxisConfiguration();

        try {
            // try to find if the axisService has already been registered
            axisService = axisConfig.getService(newServiceName);

            if (axisService == null) {
                axisService = new PetalsAxisService(newServiceName, this.componentContext,
                        consumes, extensions, this.soapServerConfig.getServicesURL(),
                        this.petalsReceiver, this.logger);

                // we set the associated service parameters
                final String serviceParameters = this.soapContext
                        .getServiceParameters(newServiceName);
                if (serviceParameters != null) {
                    try {
                        final OMElement parametersElements = this
                                .buildParametersOM(serviceParameters);

                        // get an iterator on all <parameter> children
                        final Iterator<OMElement> itr = parametersElements
                                .getChildrenWithName(new QName(DeploymentConstants.TAG_PARAMETER));

                        // iterate on parameters and set them to the associated
                        // axisService
                        while (itr.hasNext()) {
                            final OMElement parameterElement = itr.next();

                            if (DeploymentConstants.TAG_PARAMETER.equalsIgnoreCase(parameterElement
                                    .getLocalName())) {
                                axisService.addParameter(this.getParameter(parameterElement));
                            }
                        }
                    } catch (final XMLStreamException e) {
                        this.logger.warning(e.getMessage());
                    } catch (final DeploymentException e) {
                        this.logger.warning(e.getMessage());
                    }
                }

                // We prepare a list for the modules ClassLoaders
                List<ClassLoader> suModulesClassLoaders = null;

                /*
                 * Add the modules for this new service. If the services.xml
                 * file if available, the modules references can be redefined
                 */
                final List<String> modules = this.soapContext.getModules(newServiceName);
                if (modules != null) {
                    suModulesClassLoaders = new ArrayList<ClassLoader>();
                    for (final String module : modules) {
                        final AxisModule axisModule = axisConfig.getModule(module);
                        if (axisModule == null) {
                            throw new DeploymentException(Messages.getMessage(
                                    DeploymentErrorMsgs.MODULE_NOT_FOUND, module));
                        }
                        axisService.addModuleref(axisModule.getName());
                        
                        // We add the current module ClassLoader to the list
                        suModulesClassLoaders.add(axisModule.getModuleClassLoader());
                    }
                }

                final String suRootPath = ((ServiceUnitManager) this.component
                        .getServiceUnitManager()).getSUDataHandlerForConsumes(consumes)
                        .getInstallRoot();

                // We set the service ClassLoader = initial ClassLoader
                // + jars of suRootPath + modules ClassLoaders
                axisService.setClassLoader(new SoapSUClassLoader(ClassLoaderUtil
                        .getUrls(suRootPath), suModulesClassLoaders, axisService.getClassLoader()));

                // Try to load axis configuration from services.xml
                if (serviceDescriptor != null) {
                    try {
                        final FileInputStream fis = new FileInputStream(serviceDescriptor);
                        final ServiceBuilder builder = new ServiceBuilder(fis,
                                this.soapContext.getAxis2ConfigurationContext(), axisService);
                        builder.populateService(builder.buildOM());
                    } catch (final XMLStreamException e) {
                        this.logger.warning(e.getMessage());
                    } catch (final FileNotFoundException e) {
                        this.logger.warning(e.getMessage());
                    } catch (final DeploymentException e) {
                        this.logger.warning(e.getMessage());
                    }
                }

                // populate service with service descriptor
                final QName jbiServiceQName = consumes.getServiceName();
                if (jbiServiceQName != null) {
                    axisService.setTargetNamespace(jbiServiceQName.getNamespaceURI());
                }
                axisService.addParameter(new Parameter(Constants.SERVICE_CLASS, "PetalsReceiver"));
                axisService.addParameter(new Parameter(
                        Constants.Configuration.SEND_STACKTRACE_DETAILS_WITH_FAULTS, Boolean.TRUE));

                axisConfig.addService(axisService);
                
                // engage the modules if needed
                if (modules != null) {
                    for (final String module : modules) {
                        final AxisModule axisModule = axisConfig.getModule(module);
                        if (!axisService.isEngaged(axisModule)) {
                            axisService.engageModule(axisModule);
                        }
                    }
                }
                
            } else {
                this.logger.log(Level.WARNING, "The service '" + axisService.getName()
                        + "' is already registered in Axis, you can not register it twice");
            }
        } catch (final AxisFault e) {
            throw new PEtALSCDKException("Can not register Service into Axis context", e);
        }

        this.logger.log(Level.INFO, "The Axis2 service '" + newServiceName
                + "' has been registered and is available at '"
                + this.soapServerConfig.getServicesURL() + "/" + newServiceName + "'");
    }

    /**
     * Process the parameterElement object from the OM, and returns the
     * corresponding Parameter.
     * 
     * @param parameterElement
     *            <code>OMElement</code>
     * @return the Parameter parsed
     * @throws DeploymentException
     *             if bad paramName
     */
    private Parameter getParameter(final OMElement parameterElement) throws DeploymentException {

        final Parameter parameter = new Parameter();
        // setting parameterElement
        parameter.setParameterElement(parameterElement);
        // setting parameter Name
        final OMAttribute paramName = parameterElement.getAttribute(new QName(
                DeploymentConstants.ATTRIBUTE_NAME));
        if (paramName == null) {
            throw new DeploymentException(Messages.getMessage(
                    DeploymentErrorMsgs.BAD_PARAMETER_ARGUMENT, parameterElement.toString()));
        }
        parameter.setName(paramName.getAttributeValue());
        // setting parameter Value (the child element of the parameter)
        final OMElement paramValue = parameterElement.getFirstElement();
        if (paramValue != null) {
            parameter.setValue(paramValue);
            parameter.setParameterType(Parameter.OM_PARAMETER);
        } else {
            final String paratextValue = parameterElement.getText();

            parameter.setValue(paratextValue);
            parameter.setParameterType(Parameter.TEXT_PARAMETER);
        }

        return parameter;
    }

    /**
     * Creates the OMElement corresponding to the parameters String, included in
     * parameters tags.
     * 
     * @param parameters
     *            the parameters
     * @return Returns <code>OMElement</code> .
     * @throws javax.xml.stream.XMLStreamException
     * 
     */
    private final OMElement buildParametersOM(String parameters) throws XMLStreamException {
        if (parameters != null) {
            parameters = "<parameters>" + parameters + "</parameters>";
        }
        final OMElement element = (OMElement) XMLUtils.toOM(new StringReader(parameters));
        element.build();
        return element;
    }

    /**
     * Unregister the service from Axis.
     * 
     * @param serviceQName
     * @throws PEtALSComponentSDKException
     */
    private void unregisterAxisService(final ConfigurationExtensions extensions)
            throws PEtALSCDKException {

        final String endPointName = SUPropertiesHelper.getAddress(extensions);

        final AxisConfiguration axisConfig = this.soapContext.getAxis2ConfigurationContext()
                .getAxisConfiguration();

        this.logger.log(Level.INFO, "Removing Axis service '" + endPointName + "'");

        try {
            // register an axis service to axis engine
            final AxisService axisService = axisConfig.getService(endPointName);

            // Feature request : #306664
            // FIXME : We do not have to remove the service group. It is
            // temporary until the next version of Axis2 where the removeService
            // method will fix it.
            if (axisService != null) {
                axisConfig.removeServiceGroup(endPointName);
                axisService.getAxisConfiguration().removeService(endPointName);
            } else {
                this.logger.log(Level.WARNING, "Service '" + endPointName
                        + "' not found, can not be unregistered from Axis2");
            }
        } catch (final AxisFault e) {
            throw new PEtALSCDKException("Can not remove service from Axis context", e);
        }
    }

    /**
     * Store the policy path
     * @param extensions
     */
    private void setPolicyPath(final String suRootPath, final ConfigurationExtensions extensions) {
        final String address = SUPropertiesHelper.getAddress(extensions);
        final String policyPath = extensions.get(POLICY_PATH);
        
        if (policyPath == null) {
            return;
        }
        
        File path = new File(suRootPath, policyPath);
        if (logger.isLoggable(Level.FINE)) {
            logger.fine("Setting policy path for SU with address " + address + " to "
                    + path.getAbsolutePath());
        }

        if (path.exists() && path.isDirectory()) {
            this.soapContext.setPolicyPath(address, path);
        } else {
            logger.warning("The policy path does not exists, it will be ignored " + path.getAbsolutePath());
        }
    }

    /**
     * Set the service class loader 
     * @param suRootPath
     * @param extensions
     */
    private void setServiceClassloader(final String suRootPath, final ConfigurationExtensions extensions) {
        final String address = SUPropertiesHelper.getAddress(extensions);
        this.soapContext.setClassLoader(address, ClassLoaderUtil.createClassLoader(suRootPath,
                Thread.currentThread().getContextClassLoader()));
    }

    /**
     * Clean the context
     * 
     * @param serviceUnitName
     * @param descriptor
     */
    private void cleanServiceContexts(final String serviceUnitName, final Jbi descriptor) {
        this.soapContext.removeJbiDescriptor(serviceUnitName);
        this.soapContext.removeServiceDescriptor(serviceUnitName);
        
        final List<Provides> providesList = descriptor.getServices().getProvides();
        ServiceUnitDataHandler suDatahandler = null;

        for (final Provides provides : providesList) {
            if (suDatahandler == null) {
                suDatahandler = ((ServiceUnitManager) this.component.getServiceUnitManager())
                        .getSUDataHandlerForService(provides);
            }
            
            final String address = SUPropertiesHelper.getAddress(suDatahandler
                    .getConfigurationExtensions(provides));
            if (this.logger.isLoggable(Level.FINE)) {
                this.logger.fine("Cleaning context for " + address);
            }
            this.soapContext.deleteServiceContext(address);
        }
        
        final List<Consumes> consumesList = descriptor.getServices().getConsumes();
        for (final Consumes consumes : consumesList) {
            if (suDatahandler == null) {
                suDatahandler = ((ServiceUnitManager) this.component.getServiceUnitManager())
                        .getSUDataHandlerForService(consumes);
            }
            final String address = SUPropertiesHelper.getAddress(suDatahandler
                    .getConfigurationExtensions(consumes));
            if (this.logger.isLoggable(Level.FINE)) {
                this.logger.fine("Cleaning context for " + address);
            }
            this.soapContext.deleteServiceContext(address);
        }
    }

    /**
     * Add modules to the context.
     * 
     * @param provides
     */
    private void addModules(final ConfigurationExtensions extensions) {
        final String address = SUPropertiesHelper.getAddress(extensions);

        // get modules from extension
        final String token = extensions.get(MODULES);

        if (token != null) {
            // get individual modules values
            final StringTokenizer st = new StringTokenizer(token, ",");
            final List<String> mods = new ArrayList<String>(st.countTokens() + 1);
            mods.add(Constants.MODULE_ADDRESSING);
            while (st.hasMoreTokens()) {
                mods.add(st.nextToken().trim());
            }

            this.soapContext.addModules(address, mods);
        }
    }

    /**
     * Add parameters for the associated service to the componentContext.
     * 
     * @param extensions
     */
    private void setServiceParameters(final ConfigurationExtensions extensions) {

        final String address = SUPropertiesHelper.getAddress(extensions);

        // get serviceParameters from extension
        final String serviceParameters = extensions.get(SERVICE_PARAMETERS);

        if ((address != null) && (serviceParameters != null)) {
            this.soapContext.setServicePamameters(address, serviceParameters);
        }
    }

    /**
     * Create a new topic on SU deployment
     * 
     * @param topicName
     */
    protected void createTopic(final ConfigurationExtensions extensions) {

        final String address = SUPropertiesHelper.getAddress(extensions);
        final String mode = SUPropertiesHelper.getMode(extensions);

        final WsnManager notificationManager = this.soapContext.getWsnManager();

        if ("TOPIC".equals(mode)) {
            Topic topic = null;
            this.logger.info("Creating new topic '" + address + "'");
            final QName topicName = QName.valueOf(address);
            try {
                // add topic
                topic = notificationManager.addTopic(topicName);

            } catch (final WsnFault e) {
                this.logger.warning("Can not create topic : " + e.getMessage());
            }

            if (topic != null) {
                try {
                    notificationManager.reloadSubscriptions(topicName);
                } catch (final Exception e) {
                    this.logger.warning(e.getMessage());
                }
            }
        }
    }

    /**
     * Delete the topic on SU undeployment
     * 
     * @param topicName
     */
    protected void deleteTopic(final ConfigurationExtensions extensions) {
        final String name = SUPropertiesHelper.getAddress(extensions);
        final String mode = SUPropertiesHelper.getMode(extensions);

        if ("TOPIC".equals(mode)) {
            final WsnManager notificationManager = this.soapContext.getWsnManager();

            final QName topicName = QName.valueOf(name);
            this.logger.info("Deleting topic '" + topicName + "'");
            final Topic topic = notificationManager.getTopic(topicName);
            if (topic != null) {
                try {
                    notificationManager.deleteTopic(topicName);
                } catch (final WsnFault e) {
                    this.logger.warning("Fault while deleting topic : " + e.getMessage());
                }
            }
        }
    }

    /**
     * Unregister a topic
     * 
     * @param provides
     */
    protected void unregisterTopic(final ConfigurationExtensions extensions) {
        final String name = SUPropertiesHelper.getAddress(extensions);
        final String mode = SUPropertiesHelper.getMode(extensions);

        if ("TOPIC".equals(mode)) {
            this.logger.info("Unregister topic " + name);
            final WsnManager notificationManager = this.soapContext.getWsnManager();
            final QName topicName = QName.valueOf(name);
            notificationManager.cleanTopic(topicName);
        }
    }
}
