/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.binding.soap.listener;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;

import static org.ow2.petals.binding.soap.Constants.Component.NS_PREFIX;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.ADDRESS;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.DEFAULT_MODE;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.HEADERS_FILTER;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.INJECT_HEADERS;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.MODE;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.POLICY_PATH;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.REMOVE_ROOT;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.REST_ADD_NS_PREFIX;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.REST_ADD_NS_URI;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.REST_REMOVE_NS_PREFIX_RESP;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.TIMEOUT;

/**
 * @author chamerling - eBM Websourcing
 * 
 */
public class SUPropertiesHelper {

    /**
     * Creates a new instance if {@link SUPropertiesHelper}
     */
    protected SUPropertiesHelper() {
        super();
    }

    /**
     * Retrieve the timeout extensions value.
     * 
     * @param extensions
     * @return
     */
    public final static long retrieveTimeout(final ConfigurationExtensions extensions) {
        long timeout = -1L;

        if (extensions.get(TIMEOUT) != null) {
            timeout = Long.parseLong(extensions.get(TIMEOUT));
        }

        return timeout;
    }

    /**
     * Retrieve the remove root element extension value
     * 
     * @param extensions
     * @return
     */
    public final static boolean retrieveRemoveRootElement(final ConfigurationExtensions extensions) {
        return Boolean.parseBoolean(extensions.get(REMOVE_ROOT));
    }

    /**
     * Get the URI which has to be added to the message payload
     * 
     * @param props
     * @return
     */
    public static String retrieveRESTAddNSURIOnRequest(final ConfigurationExtensions extensions) {
        return extensions.get(REST_ADD_NS_URI);
    }

    /**
     * Get the prefix to be used
     * 
     * @param props
     * @return
     */
    public static String retrieveRESTAddNSPrefixOnRequest(final ConfigurationExtensions extensions) {
        String prefix = extensions.get(REST_ADD_NS_PREFIX);
        if (prefix == null) {
            prefix = NS_PREFIX;
        }
        return prefix;
    }

    /**
     * Get all the prefix which has to be removed on REST response.
     * 
     * @param props
     * @return a list of prefixes to remove or null if the option has not been
     *         set
     */
    public static List<String> retrieveRESTRemovePrefixOnResponse(
            final ConfigurationExtensions extensions) {
        List<String> result = null;
        final String tmp = extensions.get(REST_REMOVE_NS_PREFIX_RESP);
        if (tmp != null) {
            result = new ArrayList<String>();
            final StringTokenizer st = new StringTokenizer(tmp, ",");
            while (st.hasMoreTokens()) {
                result.add(st.nextToken().trim());
            }
        }
        return result;
    }

    /**
     * 
     * @param extensions
     * @return
     */
    public static List<String> retrieveHeaderList(final ConfigurationExtensions extensions) {
        List<String> result = null;
        final String tmp = extensions.get(HEADERS_FILTER);
        if (tmp != null) {
            result = new ArrayList<String>();
            final StringTokenizer st = new StringTokenizer(tmp, ",");
            while (st.hasMoreTokens()) {
                result.add(st.nextToken().trim());
            }
        }
        return result;
    }

    /**
     * We inject the headers by default
     * 
     * @param extensions
     * @return
     */
    public static boolean retrieveInjectHeader(final ConfigurationExtensions extensions) {
        boolean result = true;
        String tmp = extensions.get(INJECT_HEADERS);
        if (tmp != null) {
            result = Boolean.parseBoolean(tmp.trim());
        }
        return result;
    }

    /**
     * Get the mode. SOAP is default when not found.
     * 
     * @param extensions
     * @return
     */
    public static String getMode(final ConfigurationExtensions extensions) {
        String result = extensions.get(MODE);
        if (result == null) {
            result = DEFAULT_MODE;
        }
        return result.trim();
    }

    /**
     * Get the address
     * 
     * @param extensions
     * @return
     */
    public static String getAddress(final ConfigurationExtensions extensions) {
        return extensions.get(ADDRESS).trim();
    }
    
    /**
     * The policy is enabled if the policy path has been defined
     *  
     * @param extensions
     * @return
     */
    public static boolean isPolicyEnabled(final ConfigurationExtensions extensions) {
        String tmp = extensions.get(POLICY_PATH);
        return (tmp != null && tmp.trim().length() > 0);
    }
}
